<?php
namespace App\Entity;

use ApiPlatform\Metadata\ApiProperty;
use App\Validator\CharacterStats;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping\Column;
use Doctrine\ORM\Mapping\Entity;
use Doctrine\ORM\Mapping\JoinColumn;
use Doctrine\ORM\Mapping\ManyToMany;
use Doctrine\ORM\Mapping\ManyToOne;
use Doctrine\ORM\Mapping\Table;
use Symfony\Component\Serializer\Attribute\Groups;
use Symfony\Component\Validator\Mapping\ClassMetadata;

#[Entity]
#[Table(name: '`character`')]
class Character extends Thing
{

    #[ManyToOne(inversedBy: 'members')]
    #[JoinColumn(onDelete: 'cascade')]
    #[Groups('public')]
    #[ApiProperty(readableLink: false, writableLink: false)]
    public ?Dojo $dojo;

    #[Column(length: 64)]
    #[Groups('public')]
    public string $name;

    #[Column]
    #[Groups('detail')]
    public int $strength;

    #[Column]
    #[Groups('detail')]
    public int $constitution;

    #[Column]
    #[Groups('detail')]
    public int $agility;

    #[Column]
    #[Groups('detail')]
    public int $chi;

    #[Groups('detail')]
    #[ManyToMany(targetEntity: Technique::class, cascade: [
        'persist'
    ])]
    #[ApiProperty(readableLink: false, writableLink: false)]
    public Collection $techniques;

    public function __construct()
    {
        $this->techniques = new ArrayCollection();
    }

    /**
     * MVP: Only
     */
    #[Groups('detail')]
    public function getFreeSkillPoints()
    {
        $available = 32;
        $costs = self::skillCosts($this->getStrength());
        $costs += self::skillCosts($this->getConstitution());
        $costs += self::skillCosts($this->getAgility());
        $costs += self::skillCosts($this->getChi());

        foreach ($this->getTechniques() as $tech) {
            $costs += $tech->getCosts();
        }
        return $available - $costs;
    }

    public static function loadValidatorMetadata(ClassMetadata $metadata): void
    {
        $metadata->addConstraint(new CharacterStats());
    }

    /**
     * Calculates the aged based on the ulid value?
     */
    #[Groups('public')]
    public function getAge(): int
    {
        return 21;
    }

    public static function skillCosts(int $value)
    {
        return pow(2, $value);
    }

    public function getName(): ?string
    {
        return $this->name;
    }

    public function setName(string $name): static
    {
        $this->name = $name;

        return $this;
    }

    public function getStrength(): ?int
    {
        return $this->strength;
    }

    public function setStrength(int $strength): static
    {
        $this->strength = $strength;

        return $this;
    }

    public function getConstitution(): ?int
    {
        return $this->constitution;
    }

    public function setConstitution(int $constitution): static
    {
        $this->constitution = $constitution;

        return $this;
    }

    public function getAgility(): ?int
    {
        return $this->agility;
    }

    public function setAgility(int $agility): static
    {
        $this->agility = $agility;

        return $this;
    }

    public function getChi(): ?int
    {
        return $this->chi;
    }

    public function setChi(int $chi): static
    {
        $this->chi = $chi;

        return $this;
    }

    public function getDojo(): ?Dojo
    {
        return $this->dojo;
    }

    public function setDojo(?Dojo $dojo): static
    {
        $this->dojo = $dojo;

        return $this;
    }

    /**
     * @return Collection<int, Technique>
     */
    public function getTechniques(): Collection
    {
        return $this->techniques;
    }

    public function addTechnique(Technique $technique): static
    {
        if (!$this->techniques->contains($technique)) {
            $this->techniques->add($technique);
        }

        return $this;
    }

    public function removeTechnique(Technique $technique): static
    {
        $this->techniques->removeElement($technique);

        return $this;
    }
}

