<?php
namespace App\Entity;

use ApiPlatform\Metadata\ApiProperty;
use Doctrine\ORM\Mapping\Column;
use Doctrine\ORM\Mapping\Entity;
use Doctrine\ORM\Mapping\JoinColumn;
use Doctrine\ORM\Mapping\OneToOne;
use Doctrine\ORM\Mapping\Table;
use Symfony\Component\Security\Core\User\UserInterface;

#[Entity(repositoryClass: 'App\Repository\UserRepository')]
#[Table(name: '`user`')]
class User extends Thing implements UserInterface
{

    // from discord
    #[ApiProperty(writable: false)]
    #[Column(length: 32)]
    public string $authName;

    #[ApiProperty(writable: false)]
    #[OneToOne(inversedBy: 'owner')]
    #[JoinColumn(onDelete: 'cascade', nullable: true)]
    public ?Dojo $dojo;

    // anonymous data used for the client
    #[ApiProperty()]
    #[Column(type: 'json')]
    public mixed $properties;

    public function __construct(string $authName)
    {
        $this->authName = $authName;
        $this->properties = [];
    }

    public function getProperties(): array
    {
        return $this->properties;
    }

    public function setProperties(array $properties): static
    {
        $this->properties = $properties;

        return $this;
    }

    public function getDojo(): ?Dojo
    {
        return $this->dojo;
    }

    public function setDojo(?Dojo $dojo): static
    {
        $this->dojo = $dojo;

        return $this;
    }

    public function getUserIdentifier(): string
    {
        return $this->id;
    }

    public function eraseCredentials(): void
    {
        return;
    }

    public function getRoles(): array
    {
        $array = [
            "ROLE_USER"
        ];

        if ($this->authName == 'dehecht' || $this->authName == '.radiskull') {
            $array[] = 'ROLE_ADMIN';
        }

        return $array;
    }

    public function getAuthName(): ?string
    {
        return $this->authName;
    }

    public function setAuthName(string $authName): static
    {
        $this->authName = $authName;

        return $this;
    }
}

