<?php
namespace App\Factory;

use App\Entity\Character;
use Zenstruck\Foundry\ModelFactory;

/**
 *
 * @extends ModelFactory<Character>
 *
 * @method Character|Proxy create(array|callable $attributes = [])
 * @method static Character|Proxy createOne(array $attributes = [])
 * @method static Character|Proxy find(object|array|mixed $criteria)
 * @method static Character|Proxy findOrCreate(array $attributes)
 * @method static Character|Proxy first(string $sortedField = 'id')
 * @method static Character|Proxy last(string $sortedField = 'id')
 * @method static Character|Proxy random(array $attributes = [])
 * @method static Character|Proxy randomOrCreate(array $attributes = [])
 * @method static EntityRepository|RepositoryProxy repository()
 * @method static Character[]|Proxy[] all()
 * @method static Character[]|Proxy[] createMany(int $number, array|callable $attributes = [])
 * @method static Character[]|Proxy[] createSequence(iterable|callable $sequence)
 * @method static Character[]|Proxy[] findBy(array $attributes)
 * @method static Character[]|Proxy[] randomRange(int $min, int $max, array $attributes = [])
 * @method static Character[]|Proxy[] randomSet(int $number, array $attributes = [])
 */
final class CharacterFactory extends ModelFactory
{

    /**
     *
     * @see https://symfony.com/bundles/ZenstruckFoundryBundle/current/index.html#factories-as-services
     *
     * @todo inject services if required
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     *
     * @see https://symfony.com/bundles/ZenstruckFoundryBundle/current/index.html#model-factories
     *
     * @todo add your default values here
     */
    protected function getDefaults(): array
    {
        return [
            'name' => self::faker()->firstName(),
            'strength' => self::faker()->numberBetween(1, 4),
            'constitution' => self::faker()->numberBetween(1, 4),
            'agility' => self::faker()->numberBetween(1, 4),
            'chi' => self::faker()->numberBetween(1, 4),
            'techniques' => TechniqueFactory::createMany(2)
        ];
    }

    /**
     *
     * @see https://symfony.com/bundles/ZenstruckFoundryBundle/current/index.html#initialization
     */
    protected function initialize(): self
    {
        return $this;
        // ->afterInstantiate(function(Character $character): void {})
    }

    protected static function getClass(): string
    {
        return Character::class;
    }
}
