<?php
namespace App\Factory;

use App\Entity\Technique;
use Zenstruck\Foundry\ModelFactory;

/**
 *
 * @extends ModelFactory<Technique>
 *
 * @method Technique|Proxy create(array|callable $attributes = [])
 * @method static Technique|Proxy createOne(array $attributes = [])
 * @method static Technique|Proxy find(object|array|mixed $criteria)
 * @method static Technique|Proxy findOrCreate(array $attributes)
 * @method static Technique|Proxy first(string $sortedField = 'id')
 * @method static Technique|Proxy last(string $sortedField = 'id')
 * @method static Technique|Proxy random(array $attributes = [])
 * @method static Technique|Proxy randomOrCreate(array $attributes = [])
 * @method static EntityRepository|RepositoryProxy repository()
 * @method static Technique[]|Proxy[] all()
 * @method static Technique[]|Proxy[] createMany(int $number, array|callable $attributes = [])
 * @method static Technique[]|Proxy[] createSequence(iterable|callable $sequence)
 * @method static Technique[]|Proxy[] findBy(array $attributes)
 * @method static Technique[]|Proxy[] randomRange(int $min, int $max, array $attributes = [])
 * @method static Technique[]|Proxy[] randomSet(int $number, array $attributes = [])
 */
final class TechniqueFactory extends ModelFactory
{

    /**
     *
     * @see https://symfony.com/bundles/ZenstruckFoundryBundle/current/index.html#factories-as-services
     *
     * @todo inject services if required
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     *
     * @see https://symfony.com/bundles/ZenstruckFoundryBundle/current/index.html#model-factories
     *
     * @todo add your default values here
     */
    protected function getDefaults(): array
    {
        return [
            'name' => self::faker()->ean13(),
            'accuracy' => self::faker()->text(),
            'costs' => self::faker()->numberBetween(1, 2),
            'damage' => self::faker()->text(),
            'energy' => self::faker()->text(),
            'prerequisite' => self::faker()->boolean() ? NULL : self::new()
        ];
    }

    /**
     *
     * @see https://symfony.com/bundles/ZenstruckFoundryBundle/current/index.html#initialization
     */
    protected function initialize(): self
    {
        return $this;
        // ->afterInstantiate(function(Technique $technique): void {})
    }

    protected static function getClass(): string
    {
        return Technique::class;
    }
}
