<?php
namespace App\State;

use ApiPlatform\Metadata\Operation;
use ApiPlatform\State\ProcessorInterface;
use App\Entity\TournamentRegistration;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Component\DependencyInjection\Attribute\Autowire;
use Symfony\Component\HttpFoundation\Exception\BadRequestException;

class TournamentRegistrationStateProcessor implements ProcessorInterface
{

    public function __construct(
        #[Autowire(service: 'api_platform.doctrine.orm.state.persist_processor')] private ProcessorInterface $persistProcessor,
        private EntityManagerInterface $em)
    {}

    public function process(mixed $data, Operation $operation, array $uriVariables = [], array $context = []): TournamentRegistration
    {
        $this->updateData($data);
        $result = $this->persistProcessor->process($data, $operation, $uriVariables, $context);

        return $result;
    }

    private function updateData(TournamentRegistration $registration): void
    {
        $tournament = $registration->tournament;
        $character = $registration->character;

        if ($tournament->getCharacters()->contains($character)) {
            throw new BadRequestException("Character is already registered!");
        }

        $registration->tournament->addCharacter($registration->character);
        $this->em->persist($registration->tournament);
        $this->em->persist($registration->character);
        $this->em->flush();
    }
}
