<?php
namespace App\Validator;

use App\Entity\Character;
use Symfony\Bundle\SecurityBundle\Security;
use Symfony\Component\Validator\Constraint;
use Symfony\Component\Validator\ConstraintValidator;
use Symfony\Component\Validator\Exception\UnexpectedValueException;

class CharacterOwnedValidator extends ConstraintValidator
{

    public function __construct(private Security $security)
    {}

    public function validate($character, Constraint $constraint): void
    {
        if (! $character instanceof Character) {
            throw new UnexpectedValueException($character, Character::class);
        }

        if (! $constraint instanceof CharacterOwned) {
            throw new UnexpectedValueException($constraint, CharacterOwned::class);
        }

        if ($character->getDojo()
            ->getOwner()
            ->getId() != $this->security->getUser()->getUserIdentifier()) {
            $this->context->buildViolation($constraint->invalidOwnerMessage)->addViolation();
        }
    }
}

